@@ -28,14 +28,16 @@
 package org.apache.hc.client5.http.impl.classic;
 
 import java.io.Closeable;
-import java.io.IOException;
 import java.net.ProxySelector;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.LinkedHashMap;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.function.Function;
 
 import org.apache.hc.client5.http.AuthenticationStrategy;
 import org.apache.hc.client5.http.ConnectionKeepAliveStrategy;
@@ -66,31 +68,29 @@ import org.apache.hc.client5.http.impl.IdleConnectionEvictor;
 import org.apache.hc.client5.http.impl.NoopUserTokenHandler;
 import org.apache.hc.client5.http.impl.auth.BasicCredentialsProvider;
 import org.apache.hc.client5.http.impl.auth.BasicSchemeFactory;
+import org.apache.hc.client5.http.impl.auth.BearerSchemeFactory;
 import org.apache.hc.client5.http.impl.auth.DigestSchemeFactory;
-import org.apache.hc.client5.http.impl.auth.KerberosSchemeFactory;
-import org.apache.hc.client5.http.impl.auth.NTLMSchemeFactory;
-import org.apache.hc.client5.http.impl.auth.SPNegoSchemeFactory;
 import org.apache.hc.client5.http.impl.auth.SystemDefaultCredentialsProvider;
 import org.apache.hc.client5.http.impl.io.PoolingHttpClientConnectionManagerBuilder;
 import org.apache.hc.client5.http.impl.routing.DefaultProxyRoutePlanner;
 import org.apache.hc.client5.http.impl.routing.DefaultRoutePlanner;
 import org.apache.hc.client5.http.impl.routing.SystemDefaultRoutePlanner;
 import org.apache.hc.client5.http.io.HttpClientConnectionManager;
+import org.apache.hc.client5.http.protocol.HttpClientContext;
 import org.apache.hc.client5.http.protocol.RedirectStrategy;
 import org.apache.hc.client5.http.protocol.RequestAddCookies;
-import org.apache.hc.client5.http.protocol.RequestAuthCache;
 import org.apache.hc.client5.http.protocol.RequestClientConnControl;
 import org.apache.hc.client5.http.protocol.RequestDefaultHeaders;
 import org.apache.hc.client5.http.protocol.RequestExpectContinue;
+import org.apache.hc.client5.http.protocol.RequestUpgrade;
+import org.apache.hc.client5.http.protocol.RequestValidateTrace;
 import org.apache.hc.client5.http.protocol.ResponseProcessCookies;
 import org.apache.hc.client5.http.routing.HttpRoutePlanner;
 import org.apache.hc.core5.annotation.Internal;
 import org.apache.hc.core5.http.ConnectionReuseStrategy;
 import org.apache.hc.core5.http.Header;
 import org.apache.hc.core5.http.HttpHost;
-import org.apache.hc.core5.http.HttpRequest;
 import org.apache.hc.core5.http.HttpRequestInterceptor;
-import org.apache.hc.core5.http.HttpResponse;
 import org.apache.hc.core5.http.HttpResponseInterceptor;
 import org.apache.hc.core5.http.config.Lookup;
 import org.apache.hc.core5.http.config.NamedElementChain;
@@ -230,6 +230,7 @@ public class HttpClientBuilder {
     private boolean authCachingDisabled;
     private boolean connectionStateDisabled;
     private boolean defaultUserAgentDisabled;
+    private ProxySelector proxySelector;
 
     private List<Closeable> closeables;
 
@@ -242,7 +243,9 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link HttpRequestExecutor} instance.
+     * Sets {@link HttpRequestExecutor} instance.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setRequestExecutor(final HttpRequestExecutor requestExec) {
         this.requestExec = requestExec;
@@ -250,7 +253,9 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link HttpClientConnectionManager} instance.
+     * Sets {@link HttpClientConnectionManager} instance.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setConnectionManager(
             final HttpClientConnectionManager connManager) {
@@ -269,6 +274,7 @@ public class HttpClientBuilder {
      *
      * @param shared defines whether or not the connection manager can be shared
      *  by multiple clients.
+     * @return this instance.
      *
      * @since 4.4
      */
@@ -279,7 +285,9 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link ConnectionReuseStrategy} instance.
+     * Sets {@link ConnectionReuseStrategy} instance.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setConnectionReuseStrategy(
             final ConnectionReuseStrategy reuseStrategy) {
@@ -288,7 +296,9 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link ConnectionKeepAliveStrategy} instance.
+     * Sets {@link ConnectionKeepAliveStrategy} instance.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setKeepAliveStrategy(
             final ConnectionKeepAliveStrategy keepAliveStrategy) {
@@ -297,8 +307,10 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link AuthenticationStrategy} instance for target
+     * Sets {@link AuthenticationStrategy} instance for target
      * host authentication.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setTargetAuthenticationStrategy(
             final AuthenticationStrategy targetAuthStrategy) {
@@ -307,8 +319,10 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link AuthenticationStrategy} instance for proxy
+     * Sets {@link AuthenticationStrategy} instance for proxy
      * authentication.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setProxyAuthenticationStrategy(
             final AuthenticationStrategy proxyAuthStrategy) {
@@ -317,11 +331,13 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link UserTokenHandler} instance.
+     * Sets {@link UserTokenHandler} instance.
      * <p>
      * Please note this value can be overridden by the {@link #disableConnectionState()}
      * method.
      * </p>
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setUserTokenHandler(final UserTokenHandler userTokenHandler) {
         this.userTokenHandler = userTokenHandler;
@@ -330,6 +346,8 @@ public class HttpClientBuilder {
 
     /**
      * Disables connection state tracking.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder disableConnectionState() {
         connectionStateDisabled = true;
@@ -337,7 +355,9 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link SchemePortResolver} instance.
+     * Sets {@link SchemePortResolver} instance.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setSchemePortResolver(
             final SchemePortResolver schemePortResolver) {
@@ -346,7 +366,9 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@code User-Agent} value.
+     * Sets {@code User-Agent} value.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setUserAgent(final String userAgent) {
         this.userAgent = userAgent;
@@ -354,7 +376,9 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns default request header values.
+     * Sets default request header values.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setDefaultHeaders(final Collection<? extends Header> defaultHeaders) {
         this.defaultHeaders = defaultHeaders;
@@ -363,6 +387,8 @@ public class HttpClientBuilder {
 
     /**
      * Adds this protocol interceptor to the head of the protocol processing list.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder addResponseInterceptorFirst(final HttpResponseInterceptor interceptor) {
         Args.notNull(interceptor, "Interceptor");
@@ -375,6 +401,8 @@ public class HttpClientBuilder {
 
     /**
      * Adds this protocol interceptor to the tail of the protocol processing list.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder addResponseInterceptorLast(final HttpResponseInterceptor interceptor) {
         Args.notNull(interceptor, "Interceptor");
@@ -387,6 +415,8 @@ public class HttpClientBuilder {
 
     /**
      * Adds this protocol interceptor to the head of the protocol processing list.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder addRequestInterceptorFirst(final HttpRequestInterceptor interceptor) {
         Args.notNull(interceptor, "Interceptor");
@@ -399,6 +429,8 @@ public class HttpClientBuilder {
 
     /**
      * Adds this protocol interceptor to the tail of the protocol processing list.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder addRequestInterceptorLast(final HttpRequestInterceptor interceptor) {
         Args.notNull(interceptor, "Interceptor");
@@ -411,6 +443,8 @@ public class HttpClientBuilder {
 
     /**
      * Adds this execution interceptor before an existing interceptor.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder addExecInterceptorBefore(final String existing, final String name, final ExecChainHandler interceptor) {
         Args.notBlank(existing, "Existing");
@@ -425,6 +459,8 @@ public class HttpClientBuilder {
 
     /**
      * Adds this execution interceptor after interceptor with the given name.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder addExecInterceptorAfter(final String existing, final String name, final ExecChainHandler interceptor) {
         Args.notBlank(existing, "Existing");
@@ -439,6 +475,8 @@ public class HttpClientBuilder {
 
     /**
      * Replace an existing interceptor with the given name with new interceptor.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder replaceExecInterceptor(final String existing, final ExecChainHandler interceptor) {
         Args.notBlank(existing, "Existing");
@@ -452,6 +490,8 @@ public class HttpClientBuilder {
 
     /**
      * Add an interceptor to the head of the processing list.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder addExecInterceptorFirst(final String name, final ExecChainHandler interceptor) {
         Args.notNull(name, "Name");
@@ -465,6 +505,8 @@ public class HttpClientBuilder {
 
     /**
      * Add an interceptor to the tail of the processing list.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder addExecInterceptorLast(final String name, final ExecChainHandler interceptor) {
         Args.notNull(name, "Name");
@@ -478,6 +520,8 @@ public class HttpClientBuilder {
 
     /**
      * Disables state (cookie) management.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder disableCookieManagement() {
         this.cookieManagementDisabled = true;
@@ -486,6 +530,8 @@ public class HttpClientBuilder {
 
     /**
      * Disables automatic content decompression.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder disableContentCompression() {
         contentCompressionDisabled = true;
@@ -494,6 +540,8 @@ public class HttpClientBuilder {
 
     /**
      * Disables authentication scheme caching.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder disableAuthCaching() {
         this.authCachingDisabled = true;
@@ -501,10 +549,13 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link HttpRequestRetryStrategy} instance.
+     * Sets {@link HttpRequestRetryStrategy} instance.
      * <p>
      * Please note this value can be overridden by the {@link #disableAutomaticRetries()}
      * method.
+     * </p>
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setRetryStrategy(final HttpRequestRetryStrategy retryStrategy) {
         this.retryStrategy = retryStrategy;
@@ -513,6 +564,8 @@ public class HttpClientBuilder {
 
     /**
      * Disables automatic request recovery and re-execution.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder disableAutomaticRetries() {
         automaticRetriesDisabled = true;
@@ -520,10 +573,13 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns default proxy value.
+     * Sets default proxy value.
      * <p>
      * Please note this value can be overridden by the {@link #setRoutePlanner(
      *   org.apache.hc.client5.http.routing.HttpRoutePlanner)} method.
+     * </p>
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setProxy(final HttpHost proxy) {
         this.proxy = proxy;
@@ -531,7 +587,7 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link HttpRoutePlanner} instance.
+     * Sets {@link HttpRoutePlanner} instance.
      */
     public final HttpClientBuilder setRoutePlanner(final HttpRoutePlanner routePlanner) {
         this.routePlanner = routePlanner;
@@ -539,12 +595,14 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link RedirectStrategy} instance.
+     * Sets {@link RedirectStrategy} instance.
      * <p>
      * Please note this value can be overridden by the {@link #disableRedirectHandling()}
      * method.
      * </p>
-`     */
+     *
+     * @return this instance.
+     */
     public final HttpClientBuilder setRedirectStrategy(final RedirectStrategy redirectStrategy) {
         this.redirectStrategy = redirectStrategy;
         return this;
@@ -552,6 +610,8 @@ public class HttpClientBuilder {
 
     /**
      * Disables automatic redirect handling.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder disableRedirectHandling() {
         redirectHandlingDisabled = true;
@@ -559,7 +619,9 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link ConnectionBackoffStrategy} instance.
+     * Sets {@link ConnectionBackoffStrategy} instance.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setConnectionBackoffStrategy(
             final ConnectionBackoffStrategy connectionBackoffStrategy) {
@@ -568,7 +630,9 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns {@link BackoffManager} instance.
+     * Sets {@link BackoffManager} instance.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setBackoffManager(final BackoffManager backoffManager) {
         this.backoffManager = backoffManager;
@@ -576,8 +640,10 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns default {@link CookieStore} instance which will be used for
+     * Sets default {@link CookieStore} instance which will be used for
      * request execution if not explicitly set in the client execution context.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setDefaultCookieStore(final CookieStore cookieStore) {
         this.cookieStore = cookieStore;
@@ -585,9 +651,11 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns default {@link CredentialsProvider} instance which will be used
+     * Sets default {@link CredentialsProvider} instance which will be used
      * for request execution if not explicitly set in the client execution
      * context.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setDefaultCredentialsProvider(
             final CredentialsProvider credentialsProvider) {
@@ -596,9 +664,11 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns default {@link org.apache.hc.client5.http.auth.AuthScheme} registry which will
+     * Sets default {@link org.apache.hc.client5.http.auth.AuthScheme} registry which will
      * be used for request execution if not explicitly set in the client execution
      * context.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setDefaultAuthSchemeRegistry(
             final Lookup<AuthSchemeFactory> authSchemeRegistry) {
@@ -607,12 +677,12 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns default {@link org.apache.hc.client5.http.cookie.CookieSpec} registry which will
+     * Sets default {@link org.apache.hc.client5.http.cookie.CookieSpec} registry which will
      * be used for request execution if not explicitly set in the client execution
      * context.
      *
+     * @return this instance.
      * @see CookieSpecSupport
-     *
      */
     public final HttpClientBuilder setDefaultCookieSpecRegistry(
             final Lookup<CookieSpecFactory> cookieSpecRegistry) {
@@ -622,8 +692,10 @@ public class HttpClientBuilder {
 
 
     /**
-     * Assigns a map of {@link org.apache.hc.client5.http.entity.InputStreamFactory}s
+     * Sets a map of {@link org.apache.hc.client5.http.entity.InputStreamFactory}s
      * to be used for automatic content decompression.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setContentDecoderRegistry(
             final LinkedHashMap<String, InputStreamFactory> contentDecoderMap) {
@@ -632,9 +704,11 @@ public class HttpClientBuilder {
     }
 
     /**
-     * Assigns default {@link RequestConfig} instance which will be used
+     * Sets default {@link RequestConfig} instance which will be used
      * for request execution if not explicitly set in the client execution
      * context.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder setDefaultRequestConfig(final RequestConfig config) {
         this.defaultRequestConfig = config;
@@ -644,6 +718,8 @@ public class HttpClientBuilder {
     /**
      * Use system properties when creating and configuring default
      * implementations.
+     *
+     * @return this instance.
      */
     public final HttpClientBuilder useSystemProperties() {
         this.systemProperties = true;
@@ -656,10 +732,13 @@ public class HttpClientBuilder {
      * <p>
      * One MUST explicitly close HttpClient with {@link CloseableHttpClient#close()} in order
      * to stop and release the background thread.
+     * </p>
      * <p>
      * Please note this method has no effect if the instance of HttpClient is configured to
      * use a shared connection manager.
+     * </p>
      *
+     * @return this instance.
      * @see #setConnectionManagerShared(boolean)
      * @see ConnPoolControl#closeExpired()
      *
@@ -676,9 +755,11 @@ public class HttpClientBuilder {
      * <p>
      * One MUST explicitly close HttpClient with {@link CloseableHttpClient#close()} in order
      * to stop and release the background thread.
+     * </p>
      * <p>
      * Please note this method has no effect if the instance of HttpClient is configured to
      * use a shared connection manager.
+     * </p>
      *
      * @see #setConnectionManagerShared(boolean)
      * @see ConnPoolControl#closeIdle(TimeValue)
@@ -687,6 +768,7 @@ public class HttpClientBuilder {
      * in the connection pool. Connections whose inactivity period exceeds this value will
      * get closed and evicted from the pool.
      *
+     * @return this instance.
      * @since 4.4
      */
     public final HttpClientBuilder evictIdleConnections(final TimeValue maxIdleTime) {
@@ -698,6 +780,7 @@ public class HttpClientBuilder {
     /**
      * Disables the default user agent set by this builder if none has been provided by the user.
      *
+     * @return this instance.
      * @since 4.5.7
      */
     public final HttpClientBuilder disableDefaultUserAgent() {
@@ -705,10 +788,25 @@ public class HttpClientBuilder {
         return this;
     }
 
+    /**
+     * Sets the {@link ProxySelector} that will be used to select the proxies
+     * to be used for establishing HTTP connections. If a non-null proxy selector is set,
+     * it will take precedence over the proxy settings configured in the client.
+     *
+     * @param proxySelector the {@link ProxySelector} to be used, or null to use
+     *                      the default system proxy selector.
+     * @return this instance.
+     */
+    public final HttpClientBuilder setProxySelector(final ProxySelector proxySelector) {
+        this.proxySelector = proxySelector;
+        return this;
+    }
+
     /**
      * Request exec chain customization and extension.
      * <p>
      * For internal use.
+     * </p>
      */
     @Internal
     protected void customizeExecChain(final NamedElementChain<ExecChainHandler> execChainDefinition) {
@@ -718,6 +816,7 @@ public class HttpClientBuilder {
      * Adds to the list of {@link Closeable} resources to be managed by the client.
      * <p>
      * For internal use.
+     * </p>
      */
     @Internal
     protected void addCloseable(final Closeable closeable) {
@@ -730,6 +829,11 @@ public class HttpClientBuilder {
         closeables.add(closeable);
     }
 
+    @Internal
+    protected Function<HttpContext, HttpClientContext> contextAdaptor() {
+        return HttpClientContext::castOrCreate;
+    }
+
     public CloseableHttpClient build() {
         // Create main request executor
         // We copy the instance fields to avoid changing them, and rename to avoid accidental use of the wrong version
@@ -739,7 +843,11 @@ public class HttpClientBuilder {
         }
         HttpClientConnectionManager connManagerCopy = this.connManager;
         if (connManagerCopy == null) {
-            connManagerCopy = PoolingHttpClientConnectionManagerBuilder.create().build();
+            final PoolingHttpClientConnectionManagerBuilder connectionManagerBuilder = PoolingHttpClientConnectionManagerBuilder.create();
+            if (systemProperties) {
+                connectionManagerBuilder.useSystemProperties();
+            }
+            connManagerCopy = connectionManagerBuilder.build();
         }
         ConnectionReuseStrategy reuseStrategyCopy = this.reuseStrategy;
         if (reuseStrategyCopy == null) {
@@ -748,13 +856,7 @@ public class HttpClientBuilder {
                 if ("true".equalsIgnoreCase(s)) {
                     reuseStrategyCopy = DefaultClientConnectionReuseStrategy.INSTANCE;
                 } else {
-                    reuseStrategyCopy = new ConnectionReuseStrategy() {
-                        @Override
-                        public boolean keepAlive(
-                                final HttpRequest request, final HttpResponse response, final HttpContext context) {
-                            return false;
-                        }
-                    };
+                    reuseStrategyCopy = (request, response, context) -> false;
                 }
             } else {
                 reuseStrategyCopy = DefaultClientConnectionReuseStrategy.INSTANCE;
@@ -793,17 +895,6 @@ public class HttpClientBuilder {
             }
         }
 
-        final NamedElementChain<ExecChainHandler> execChainDefinition = new NamedElementChain<>();
-        execChainDefinition.addLast(
-                new MainClientExec(connManagerCopy, reuseStrategyCopy, keepAliveStrategyCopy, userTokenHandlerCopy),
-                ChainElement.MAIN_TRANSPORT.name());
-        execChainDefinition.addFirst(
-                new ConnectExec(
-                        reuseStrategyCopy,
-                        new DefaultHttpProcessor(new RequestTargetHost(), new RequestUserAgent(userAgentCopy)),
-                        proxyAuthStrategyCopy),
-                ChainElement.CONNECT.name());
-
         final HttpProcessorBuilder b = HttpProcessorBuilder.create();
         if (requestInterceptors != null) {
             for (final RequestInterceptorEntry entry: requestInterceptors) {
@@ -820,20 +911,19 @@ public class HttpClientBuilder {
             }
         }
         b.addAll(
+                new RequestTargetHost(),
+                new RequestValidateTrace(),
                 new RequestDefaultHeaders(defaultHeaders),
                 new RequestContent(),
-                new RequestTargetHost(),
                 new RequestClientConnControl(),
                 new RequestUserAgent(userAgentCopy),
-                new RequestExpectContinue());
+                new RequestExpectContinue(),
+                new RequestUpgrade());
         if (!cookieManagementDisabled) {
-            b.add(new RequestAddCookies());
-        }
-        if (!authCachingDisabled) {
-            b.add(new RequestAuthCache());
+            b.add(RequestAddCookies.INSTANCE);
         }
         if (!cookieManagementDisabled) {
-            b.add(new ResponseProcessCookies());
+            b.add(ResponseProcessCookies.INSTANCE);
         }
         if (requestInterceptors != null) {
             for (final RequestInterceptorEntry entry: requestInterceptors) {
@@ -850,10 +940,44 @@ public class HttpClientBuilder {
             }
         }
         final HttpProcessor httpProcessor = b.build();
+
+        final NamedElementChain<ExecChainHandler> execChainDefinition = new NamedElementChain<>();
+        execChainDefinition.addLast(
+                new MainClientExec(connManagerCopy, httpProcessor, reuseStrategyCopy, keepAliveStrategyCopy, userTokenHandlerCopy),
+                ChainElement.MAIN_TRANSPORT.name());
         execChainDefinition.addFirst(
-                new ProtocolExec(httpProcessor, targetAuthStrategyCopy, proxyAuthStrategyCopy, schemePortResolver),
+                new ConnectExec(
+                        reuseStrategyCopy,
+                        new DefaultHttpProcessor(new RequestTargetHost(), new RequestUserAgent(userAgentCopy)),
+                        proxyAuthStrategyCopy,
+                        schemePortResolver != null ? schemePortResolver : DefaultSchemePortResolver.INSTANCE,
+                        authCachingDisabled),
+                ChainElement.CONNECT.name());
+
+        execChainDefinition.addFirst(
+                new ProtocolExec(
+                        targetAuthStrategyCopy,
+                        proxyAuthStrategyCopy,
+                        schemePortResolver != null ? schemePortResolver : DefaultSchemePortResolver.INSTANCE,
+                        authCachingDisabled),
                 ChainElement.PROTOCOL.name());
 
+        if (!contentCompressionDisabled) {
+            if (contentDecoderMap != null) {
+                final List<String> encodings = new ArrayList<>(contentDecoderMap.keySet());
+                final RegistryBuilder<InputStreamFactory> b2 = RegistryBuilder.create();
+                for (final Map.Entry<String, InputStreamFactory> entry: contentDecoderMap.entrySet()) {
+                    b2.register(entry.getKey(), entry.getValue());
+                }
+                final Registry<InputStreamFactory> decoderRegistry = b2.build();
+                execChainDefinition.addFirst(
+                        new ContentCompressionExec(encodings, decoderRegistry, true),
+                        ChainElement.COMPRESS.name());
+            } else {
+                execChainDefinition.addFirst(new ContentCompressionExec(true), ChainElement.COMPRESS.name());
+            }
+        }
+
         // Add request retry executor, if not disabled
         if (!automaticRetriesDisabled) {
             HttpRequestRetryStrategy retryStrategyCopy = this.retryStrategy;
@@ -873,35 +997,21 @@ public class HttpClientBuilder {
             }
             if (proxy != null) {
                 routePlannerCopy = new DefaultProxyRoutePlanner(proxy, schemePortResolverCopy);
+            } else if (this.proxySelector != null) {
+                routePlannerCopy = new SystemDefaultRoutePlanner(schemePortResolverCopy, this.proxySelector);
             } else if (systemProperties) {
-                routePlannerCopy = new SystemDefaultRoutePlanner(
-                        schemePortResolverCopy, ProxySelector.getDefault());
+                final ProxySelector defaultProxySelector = AccessController.doPrivileged((PrivilegedAction<ProxySelector>) ProxySelector::getDefault);
+                routePlannerCopy = new SystemDefaultRoutePlanner(schemePortResolverCopy, defaultProxySelector);
             } else {
                 routePlannerCopy = new DefaultRoutePlanner(schemePortResolverCopy);
             }
         }
 
-        if (!contentCompressionDisabled) {
-            if (contentDecoderMap != null) {
-                final List<String> encodings = new ArrayList<>(contentDecoderMap.keySet());
-                final RegistryBuilder<InputStreamFactory> b2 = RegistryBuilder.create();
-                for (final Map.Entry<String, InputStreamFactory> entry: contentDecoderMap.entrySet()) {
-                    b2.register(entry.getKey(), entry.getValue());
-                }
-                final Registry<InputStreamFactory> decoderRegistry = b2.build();
-                execChainDefinition.addFirst(
-                        new ContentCompressionExec(encodings, decoderRegistry, true),
-                        ChainElement.COMPRESS.name());
-            } else {
-                execChainDefinition.addFirst(new ContentCompressionExec(true), ChainElement.COMPRESS.name());
-            }
-        }
-
         // Add redirect executor, if not disabled
         if (!redirectHandlingDisabled) {
             RedirectStrategy redirectStrategyCopy = this.redirectStrategy;
             if (redirectStrategyCopy == null) {
-                redirectStrategyCopy = DefaultRedirectStrategy.INSTANCE;
+                redirectStrategyCopy = schemePortResolver != null ? new DefaultRedirectStrategy(schemePortResolver) : DefaultRedirectStrategy.INSTANCE;
             }
             execChainDefinition.addFirst(
                     new RedirectExec(routePlannerCopy, redirectStrategyCopy),
@@ -952,9 +1062,7 @@ public class HttpClientBuilder {
             authSchemeRegistryCopy = RegistryBuilder.<AuthSchemeFactory>create()
                 .register(StandardAuthScheme.BASIC, BasicSchemeFactory.INSTANCE)
                 .register(StandardAuthScheme.DIGEST, DigestSchemeFactory.INSTANCE)
-                .register(StandardAuthScheme.NTLM, NTLMSchemeFactory.INSTANCE)
-                .register(StandardAuthScheme.SPNEGO, SPNegoSchemeFactory.DEFAULT)
-                .register(StandardAuthScheme.KERBEROS, KerberosSchemeFactory.DEFAULT)
+                .register(StandardAuthScheme.BEARER, BearerSchemeFactory.INSTANCE)
                 .build();
         }
         Lookup<CookieSpecFactory> cookieSpecRegistryCopy = this.cookieSpecRegistry;
@@ -985,18 +1093,13 @@ public class HttpClientBuilder {
                 if (connManagerCopy instanceof ConnPoolControl) {
                     final IdleConnectionEvictor connectionEvictor = new IdleConnectionEvictor((ConnPoolControl<?>) connManagerCopy,
                             maxIdleTime, maxIdleTime);
-                    closeablesCopy.add(new Closeable() {
-
-                        @Override
-                        public void close() throws IOException {
-                            connectionEvictor.shutdown();
-                            try {
-                                connectionEvictor.awaitTermination(Timeout.ofSeconds(1));
-                            } catch (final InterruptedException interrupted) {
-                                Thread.currentThread().interrupt();
-                            }
+                    closeablesCopy.add(() -> {
+                        connectionEvictor.shutdown();
+                        try {
+                            connectionEvictor.awaitTermination(Timeout.ofSeconds(1));
+                        } catch (final InterruptedException interrupted) {
+                            Thread.currentThread().interrupt();
                         }
-
                     });
                     connectionEvictor.start();
                 }
@@ -1013,6 +1116,7 @@ public class HttpClientBuilder {
                 authSchemeRegistryCopy,
                 defaultCookieStore,
                 defaultCredentialsProvider,
+                contextAdaptor(),
                 defaultRequestConfig != null ? defaultRequestConfig : RequestConfig.DEFAULT,
                 closeablesCopy);
     }
